@@ -8,16 +8,29 @@ apt-get update
 # has bugs that make our tests deadlock (the relevant tests detect this and
 # disable themselves, but it means that to get full coverage we have to use
 # the openssl version).
+# The oddly-named python-software-properties includes add-apt-repository.
 APT_PACKAGES="
 python-pip
 python-dev
-python3-pycurl
 libcurl4-openssl-dev
+python-software-properties
 "
 
 apt-get -y install $APT_PACKAGES
 
-# Ubuntu 12.10 includes python 2.7 and 3.2.
+
+# Ubuntu 10.04 has python 2.6 as default; install more from here.
+add-apt-repository ppa:fkrull/deadsnakes
+apt-get update
+
+DEADSNAKES_PACKAGES="
+python2.7
+python2.7-dev
+python3.2
+python3.2-dev
+"
+apt-get -y install $DEADSNAKES_PACKAGES
+
 
 PIP_PACKAGES="
 futures
