@@ -55,6 +55,7 @@ _default_locale = "en_US"
 _translations = {}
 _supported_locales = frozenset([_default_locale])
 _use_gettext = False
+CONTEXT_SEPARATOR = "\x04"
 
 
 def get(*locale_codes):
@@ -273,6 +274,9 @@ class Locale(object):
         """
         raise NotImplementedError()
 
+    def pgettext(self, context, message, plural_message=None, count=None):
+        raise NotImplementedError()
+
     def format_date(self, date, gmt_offset=0, relative=True, shorter=False,
                     full_format=False):
         """Formats the given date (which should be GMT).
@@ -286,8 +290,6 @@ class Locale(object):
         This method is primarily intended for dates in the past.
         For dates in the future, we fall back to full format.
         """
-        if self.code.startswith("ru"):
-            relative = False
         if isinstance(date, numbers.Real):
             date = datetime.datetime.utcfromtimestamp(date)
         now = datetime.datetime.utcnow()
@@ -424,6 +426,11 @@ class CSVLocale(Locale):
             message_dict = self.translations.get("unknown", {})
         return message_dict.get(message, message)
 
+    def pgettext(self, context, message, plural_message=None, count=None):
+        if self.translations:
+            gen_log.warning('pgettext is not supported by CSVLocale')
+        return self.translate(message, plural_message, count)
+
 
 class GettextLocale(Locale):
     """Locale implementation using the `gettext` module."""
@@ -447,6 +454,44 @@ class GettextLocale(Locale):
         else:
             return self.gettext(message)
 
+    def pgettext(self, context, message, plural_message=None, count=None):
+        """Allows to set context for translation, accepts plural forms.
+
+        Usage example::
+
+            pgettext("law", "right")
+            pgettext("good", "right")
+
+        Plural message example::
+
+            pgettext("organization", "club", "clubs", len(clubs))
+            pgettext("stick", "club", "clubs", len(clubs))
+
+        To generate POT file with context, add following options to step 1
+        of `load_gettext_translations` sequence::
+
+            xgettext [basic options] --keyword=pgettext:1c,2 --keyword=pgettext:1c,2,3
+
+        .. versionadded:: 4.2
+        """
+        if plural_message is not None:
+            assert count is not None
+            msgs_with_ctxt = ("%s%s%s" % (context, CONTEXT_SEPARATOR, message),
+                          "%s%s%s" % (context, CONTEXT_SEPARATOR, plural_message),
+                          count)
+            result = self.ngettext(*msgs_with_ctxt)
+            if CONTEXT_SEPARATOR in result:
+                # Translation not found
+                result = self.ngettext(message, plural_message, count)
+            return result
+        else:
+            msg_with_ctxt = "%s%s%s" % (context, CONTEXT_SEPARATOR, message)
+            result = self.gettext(msg_with_ctxt)
+            if CONTEXT_SEPARATOR in result:
+                # Translation not found
+                result = message
+            return result
+
 LOCALE_NAMES = {
     "af_ZA": {"name_en": u("Afrikaans"), "name": u("Afrikaans")},
     "am_ET": {"name_en": u("Amharic"), "name": u('\u12a0\u121b\u122d\u129b')},
