@@ -27,7 +27,12 @@ from __future__ import absolute_import, division, print_function, with_statement
 
 import os
 
-if os.name == 'nt':
+if 'APPENGINE_RUNTIME' in os.environ:
+    from tornado.platform.common import Waker
+
+    def set_close_exec(fd):
+        pass
+elif os.name == 'nt':
     from tornado.platform.common import Waker
     from tornado.platform.windows import set_close_exec
 else:
@@ -37,9 +42,13 @@ try:
     # monotime monkey-patches the time module to have a monotonic function
     # in versions of python before 3.3.
     import monotime
+    # Silence pyflakes warning about this unused import
+    monotime
 except ImportError:
     pass
 try:
     from time import monotonic as monotonic_time
 except ImportError:
     monotonic_time = None
+
+__all__ = ['Waker', 'set_close_exec', 'monotonic_time']
